package lowlevelfunc.boolfun;

/**
 * Tests various cryptographic properties of a boolean function passed as
 * command line argument and prints them.
 * 
 * @author Luca Mariot
 * @version 1.0
 */

import java.math.BigInteger;

public class TestSingleBoolFunProp {
    
    public static void main(String[] args) {
        
        //The arguments passed must be the function number (according to
        //Wolfram's convention) and the number of variables.
        if(args.length!= 2) {
            
            System.err.println("Usage: java "
                    + "lowlevelfunc.boolfun.TestBoolFunProp func_num nvar");
            
            System.exit(1);           
            
        }
        
        BigInteger funcnum = new BigInteger(args[0]);
        int nvar = Integer.parseInt(args[1]);
        
        //Creates the set of indices to check crypto properties.
        int[][] indices = new int[nvar][];
        for(int i=0; i<nvar; i++) {
            
            indices[i] = CombTools.genBinCombs(nvar-i-1, i+1);
            
        }
        
        //Instatiate the boolean function.
        BooleanFunction boolfun = new BooleanFunction(funcnum,nvar);
        
        //Compute the cryptographic properties.
        CheckProp.computeAllCryptoProp(boolfun, indices);
        
        //Print the tables and the properties of the function.
        CheckProp.printFuncNum(boolfun);
        CheckProp.printFuncTables(boolfun);
        CheckProp.printANF(boolfun);
        CheckProp.printCryptoProp(boolfun, nvar);
        
        System.out.println("");
        
    }
    
}
