package softcomp.genalg;

/**
 * Class containing methods for various mutation operators.
 * 
 * @author Luca Mariot
 * @version 1.0
 */

import java.util.Random;
import java.util.Vector;
import lowlevelfunc.boolfun.BinTools;

public class MutationOperators {
    
    /**
     * Return the position of a value in an array.
     * 
     * @param vec array of integers
     * @param val value to be searched
     * @return the position of val in vec (-1 if val is not contained in vec)
     */
    public static int findPos(int[] vec, int val) {
        
        int i=0;
        int pos = vec.length;
        
        while((i<vec.length)) {
            
            if(vec[i] == val) {
                return i;
            }
            
            i++;
            
        }
        
        return -1;
        
    }
    
    /**
     * Switch mutation operator for Walsh spectra
     * 
     * @param indiv     array representing a Walsh spectrum
     * @param mapz      Map of zeros in indiv
     * @param mapn      Map of -sprad in indiv
     * @param mapp      Map of +sprad in indiv
     * @param pmut      mutation probability
     * @param sprad     spectral radius
     * @param genrand   Random number generator
     */
    public static void switchMutationComb(int[] indiv, int[] mapz,
            int[] mapn, int[] mapp, double pmut, int sprad,
            Random genrand) {
        
        for(int i=0; i<indiv.length; i++) {
            
            if(genrand.nextDouble() < pmut) {
                
                //System.out.println("Position "+i+" mutated");
                
                //Swap the current value with another different one, randomly chosen
                
                if(indiv[i]==0) {
                    
                    //Pick a position either set to +sprad or -sprad and swap it
                    int pos = genrand.nextInt(mapn.length+mapp.length);
                    
                    if(pos < mapn.length) {
                        
                        //Swap the value with a -sprad
                        indiv[i] = -sprad;
                        indiv[mapn[pos]] = 0;
                        
                        //Update the map of zeros and -sprad
                        int ipos = findPos(mapz, i);
                        mapz[ipos] = mapn[pos];
                        mapn[pos] = i;
                        
                    } else {
                        
                        //Swap the value with a +sprad
                        indiv[i] = sprad;
                        int realpos = pos - mapn.length;
                        indiv[mapp[realpos]] = 0;
                        
                        //Update the maps of zeros and +sprad
                        int ipos = findPos(mapz,i);
                        mapz[ipos] = mapp[realpos];
                        mapp[realpos] = i;
                        
                    }
                    
                } else {
                    
                    if(indiv[i]==-sprad) {

                        //Pick a position either set to 0 or +sprad and swap it
                        int pos = genrand.nextInt(mapz.length+mapp.length);
                        
                        if(pos<mapz.length) {
                            
                            //Swap the value with a 0
                            indiv[i] = 0;
                            indiv[mapz[pos]] = -sprad;
                            
                            //Update the maps of -sprad and zeros
                            int ipos = findPos(mapn,i);
                            mapn[ipos] = mapz[pos];
                            mapz[pos] = i;

                        } else {

                            //Swap the value with a +sprad
                            indiv[i] = sprad;
                            int realpos = pos - mapz.length;
                            indiv[mapp[realpos]] = -sprad;
                            
                            //Update the maps of -sprad and +sprad
                            int ipos = findPos(mapn,i);
                            mapn[ipos] = mapp[realpos];
                            mapp[realpos] = i;

                        }

                    } else {
                        
                        //Case indiv[i] = +sprad. Pick a position either set to
                        //0 or -sprad and swap it
                        int pos = genrand.nextInt(mapz.length+mapn.length);
                        
                        if(pos<mapz.length) {
                            
                            //Swap the value with a 0
                            indiv[i] = 0;
                            indiv[mapz[pos]] = sprad;
                            
                            //Update the maps of +sprad and zeros
                            int ipos = findPos(mapp,i);
                            mapp[ipos] = mapz[pos];
                            mapz[pos] = i;

                        } else {

                            //Swap the value with a -sprad
                            indiv[i] = -sprad;
                            int realpos = pos - mapz.length;
                            indiv[mapn[realpos]] = sprad;
                            
                            //Update the maps of -sprad and +sprad
                            int ipos = findPos(mapp,i);
                            mapp[ipos] = mapn[realpos];
                            mapn[realpos] = i;

                        }
                        
                    }
                    
                }
                
            }
            
        }
        
    }
    
}
